@@ -31,10 +31,10 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2013 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2014 Canada's Michael Smith Genome Sciences Centre\n";
 
 static const char USAGE_MESSAGE[] =
-"Usage: " PROGRAM " [OPTION]... [FILE]...\n"
+"Usage: " PROGRAM " -k<kmer> [OPTION]... [FILE]...\n"
 "Find overlaps of [m,k) bases. Contigs may be read from FILE(s)\n"
 "or standard input. Output is written to standard output.\n"
 "Overlaps of exactly k-1 bases are found using a hash table.\n"
@@ -47,6 +47,8 @@ static const char USAGE_MESSAGE[] =
 "      --adj             output the results in adj format [default]\n"
 "      --dot             output the results in dot format\n"
 "      --sam             output the results in SAM format\n"
+"      --SS              expect contigs to be oriented correctly\n"
+"      --no-SS           no assumption about contig orientation\n"
 "  -v, --verbose         display verbose output\n"
 "      --help            display this help and exit\n"
 "      --version         output version information and exit\n"
@@ -57,6 +59,9 @@ namespace opt {
 	unsigned k; // used by GraphIO
 	int format; // used by GraphIO
 
+	/** Run a strand-specific RNA-Seq assembly. */
+	static int ss;
+
 	/** The minimum required amount of overlap. */
 	static unsigned minOverlap = 50;
 }
@@ -71,6 +76,8 @@ static const struct option longopts[] = {
 	{ "adj",     no_argument,       &opt::format, ADJ },
 	{ "dot",     no_argument,       &opt::format, DOT },
 	{ "sam",     no_argument,       &opt::format, SAM },
+	{ "SS",      no_argument,       &opt::ss, 1 },
+	{ "no-SS",   no_argument,       &opt::ss, 0 },
 	{ "verbose", no_argument,       NULL, 'v' },
 	{ "help",    no_argument,       NULL, OPT_HELP },
 	{ "version", no_argument,       NULL, OPT_VERSION },
@@ -102,6 +109,8 @@ static void addOverlapsSA(Graph& g, const SuffixArray& sa,
 		pair<It, It> range = sa.equal_range(q);
 		for (It it = range.first; it != range.second; ++it) {
 			ContigNode u(it->second);
+			if (opt::ss && u.sense() != v.sense())
+				continue;
 			if (seen.insert(u).second) {
 				// Add the longest overlap between two vertices.
 				unsigned overlap = it->first.size();
@@ -118,7 +127,6 @@ static void addOverlapsSA(Graph& g, const vector<Kmer>& prefixes)
 	typedef pair<string, ContigNode> Suffix;
 	typedef vector<Suffix> Suffixes;
 	Suffixes suffixes;
-	SuffixArray sa(opt::minOverlap);
 
 	typedef graph_traits<Graph>::vertex_descriptor V;
 	typedef graph_traits<Graph>::vertex_iterator Vit;
@@ -132,8 +140,12 @@ static void addOverlapsSA(Graph& g, const vector<Kmer>& prefixes)
 		assert(uci < prefixes.size());
 		string suffix(reverseComplement(prefixes[uci]).str());
 		suffixes.push_back(Suffix(suffix, u));
-		sa.insert(suffixes.back());
 	}
+
+	SuffixArray sa(opt::minOverlap);
+	for (Suffixes::const_iterator it = suffixes.begin();
+			it != suffixes.end(); ++it)
+		sa.insert(*it);
 	sa.construct();
 
 	for (Suffixes::const_iterator it = suffixes.begin();
@@ -144,7 +156,8 @@ static void addOverlapsSA(Graph& g, const vector<Kmer>& prefixes)
 }
 
 /** An index of suffixes of k-1 bp. */
-typedef unordered_map<Kmer, vector<ContigNode>, hashKmer> SuffixMap;
+typedef unordered_map<Kmer, vector<ContigNode>, hash<Kmer> >
+	SuffixMap;
 
 /** Read contigs. Add contig properties to the graph. Add prefixes to
  * the collection and add suffixes to their index.
@@ -261,6 +274,8 @@ int main(int argc, char** argv)
 				itu = edges.begin(); itu != edges.end(); ++itu) {
 			V uc = get(vertex_complement, g, *itu);
 			V vc = get(vertex_complement, g, v);
+			if (opt::ss && uc.sense() != vc.sense())
+				continue;
 			add_edge(vc, uc, -(int)opt::k + 1, static_cast<DG&>(g));
 		}
 	}
