@@ -82,14 +82,18 @@ void loadSequences(ISequenceCollection* seqCollection, string inFile)
 	}
 
 	size_t count = 0, count_good = 0,
-			 count_small = 0, count_nonACGT = 0;
-	FastaReader reader(inFile.c_str(), FastaReader::FOLD_CASE);
+			 count_small = 0, count_nonACGT = 0,
+			 count_reversed = 0;
+	int fastaFlags = opt::maskCov ?  FastaReader::NO_FOLD_CASE :
+			FastaReader::FOLD_CASE;
+	FastaReader reader(inFile.c_str(), fastaFlags);
 	if (endsWith(inFile, ".jf") || endsWith(inFile, ".jfq")) {
 		// Load k-mer with coverage data.
 		count = loadKmer(*seqCollection, reader);
 		count_good = count;
 	} else
-	for (Sequence seq; reader >> seq;) {
+	for (FastaRecord rec; reader >> rec;) {
+		Sequence seq = rec.seq;
 		size_t len = seq.length();
 		if (opt::kmerSize > len) {
 			count_small++;
@@ -115,11 +119,24 @@ void loadSequences(ISequenceCollection* seqCollection, string inFile)
 
 		bool good = seq.find_first_not_of("ACGT0123") == string::npos;
 		bool discarded = true;
+
+		if (opt::ss && rec.id.size() > 2
+				&& rec.id.substr(rec.id.size()-2) == "/1") {
+			seq = reverseComplement(seq);
+			count_reversed++;
+		}
+
 		for (unsigned i = 0; i < len - opt::kmerSize + 1; i++) {
 			Sequence kmer(seq, i, opt::kmerSize);
-			if (good || kmer.find_first_not_of("ACGT0123")
+			if (good || kmer.find_first_not_of("acgtACGT0123")
 					== string::npos) {
-				seqCollection->add(Kmer(kmer));
+				if (good || kmer.find_first_of("acgt") == string::npos)
+					seqCollection->add(Kmer(kmer));
+				else {
+					transform(kmer.begin(), kmer.end(), kmer.begin(),
+							::toupper);
+					seqCollection->add(Kmer(kmer), 0);
+				}
 				discarded = false;
 			}
 		}
@@ -139,6 +156,9 @@ void loadSequences(ISequenceCollection* seqCollection, string inFile)
 	logger(1) << "Read " << count << " reads. ";
 	seqCollection->printLoad();
 
+	if (count_reversed > 0)
+		cerr << "`" << inFile << "': "
+			"reversed " << count_reversed << " reads\n";
 	if (count_small > 0)
 		cerr << "`" << inFile << "': "
 			"discarded " << count_small << " reads "
@@ -230,7 +250,7 @@ size_t markAmbiguous(ISequenceCollection* g)
 		if (++progress % 1000000 == 0)
 			logger(1) << "Splitting: " << progress << '\n';
 
-		if (it->first.isPalindrome()) {
+		if (!opt::ss && it->first.isPalindrome()) {
 			countv += 2;
 			g->mark(it->first);
 			counte += markNeighbours(g, *it, SENSE);
@@ -238,7 +258,7 @@ size_t markAmbiguous(ISequenceCollection* g)
 			for (extDirection sense = SENSE;
 					sense <= ANTISENSE; ++sense) {
 				if (it->second.getExtension(sense).isAmbiguous()
-						|| it->first.isPalindrome(sense)) {
+						|| (!opt::ss && it->first.isPalindrome(sense))) {
 					countv++;
 					g->mark(it->first, sense);
 					counte += markNeighbours(g, *it, sense);
@@ -737,7 +757,7 @@ bool processLinearExtensionForBranch(BranchRecord& branch,
 		unsigned maxLength, bool addKmer)
 {
 	/** Stop contig assembly at palindromes. */
-	const bool stopAtPalindromes = maxLength == UINT_MAX;
+	const bool stopAtPalindromes = !opt::ss && maxLength == UINT_MAX;
 
 	extDirection dir = branch.getDirection();
 	if (branch.isTooLong(maxLength)) {
@@ -901,7 +921,8 @@ size_t assemble(SequenceCollectionHash* seqCollection,
 					currSeq, extRec, multiplicity, UINT_MAX);
 		}
 
-		if (currBranch.isCanonical()) {
+		if ((opt::ss && currBranch.getDirection() == SENSE)
+				|| (!opt::ss && currBranch.isCanonical())) {
 			size_t removed = assembleContig(seqCollection,
 					fileWriter, currBranch, contigID++);
 			assembledKmer += currBranch.size();
