@@ -2,6 +2,7 @@
 #include "SequenceCollection.h"
 #include "Log.h"
 #include "Common/Options.h"
+#include "Assembly/Options.h"
 #include "MemoryUtil.h"
 #include "StringUtil.h" // for toSI
 #include "Timer.h"
@@ -35,24 +36,45 @@ SequenceCollectionHash::SequenceCollectionHash()
 #endif
 }
 
+/** sparse_hash_set requires that set_deleted_key()
+ * is called before calling erase(). This key cannot
+ * be an existing kmer in m_data. This function sets
+ * the deleted key and should be called after all
+ * data has been loaded.
+ */
+void SequenceCollectionHash::setDeletedKey()
+{
+#if HAVE_GOOGLE_SPARSE_HASH_MAP
+	for (SequenceDataHash::iterator it = m_data.begin();
+			it != m_data.end(); it++) {
+		Kmer rc(reverseComplement(it->first));
+		bool isrc;
+		SequenceDataHash::iterator search = find(rc, isrc);
+		// If this is false, we should have a palindrome or we're
+		// doing a SS assembly.
+		if (isrc || search == m_data.end()) {
+			m_data.set_deleted_key(rc);
+			return;
+		}
+	}
+	logger(1) << "error: unable to set deleted key.\n";
+	exit(EXIT_FAILURE);
+#else
+	return;
+#endif
+}
+
 /** Add the specified k-mer to this collection. */
 void SequenceCollectionHash::add(const Kmer& seq, unsigned coverage)
 {
 	bool rc;
 	SequenceCollectionHash::iterator it = find(seq, rc);
 	if (it == m_data.end()) {
-#if HAVE_GOOGLE_SPARSE_HASH_MAP
-		if (m_data.empty()) {
-			/* sparse_hash_set requires that set_deleted_key()
-			 * is called before calling erase(). */
-			Kmer rc(reverseComplement(seq));
-			assert(rc != seq);
-			m_data.set_deleted_key(rc);
-		}
-#endif
 		m_data.insert(make_pair(seq, KmerData(SENSE, coverage)));
-	} else
+	} else if (coverage > 0) {
+		assert(!rc || !opt::ss);
 		it->second.addMultiplicity(rc ? ANTISENSE : SENSE, coverage);
+	}
 }
 
 /** Clean up by erasing sequences flagged as deleted.
@@ -89,11 +111,16 @@ bool SequenceCollectionHash::setBaseExtension(
 	SequenceCollectionHash::iterator it = find(kmer, rc);
 	if (it == m_data.end())
 		return false;
-	bool palindrome = kmer.isPalindrome();
-	if (!rc || palindrome)
+	if (opt::ss) {
+		assert(!rc);
 		it->second.setBaseExtension(dir, base);
-	if (rc || palindrome)
-		it->second.setBaseExtension(!dir, complementBaseCode(base));
+	} else {
+		bool palindrome = kmer.isPalindrome();
+		if (!rc || palindrome)
+			it->second.setBaseExtension(dir, base);
+		if (rc || palindrome)
+			it->second.setBaseExtension(!dir, complementBaseCode(base));
+	}
 	return true;
 }
 
@@ -104,11 +131,16 @@ void SequenceCollectionHash::removeExtension(const Kmer& kmer,
 	bool rc;
 	SequenceCollectionHash::iterator it = find(kmer, rc);
 	assert(it != m_data.end());
-	bool palindrome = kmer.isPalindrome();
-	if (!rc || palindrome)
+	if (opt::ss) {
+		assert(!rc);
 		it->second.removeExtension(dir, ext);
-	if (rc || palindrome)
-		it->second.removeExtension(!dir, ~ext);
+	} else {
+		bool palindrome = kmer.isPalindrome();
+		if (!rc || palindrome)
+			it->second.removeExtension(dir, ext);
+		if (rc || palindrome)
+			it->second.removeExtension(!dir, ~ext);
+	}
 	notify(*it);
 }
 
@@ -144,7 +176,7 @@ SequenceCollectionHash::iterator SequenceCollectionHash::find(
 		const Kmer& key, bool& rc)
 {
 	SequenceCollectionHash::iterator it = find(key);
-	if (it != m_data.end()) {
+	if (opt::ss || it != m_data.end()) {
 		rc = false;
 		return it;
 	} else {
@@ -160,7 +192,7 @@ SequenceCollectionHash::const_iterator SequenceCollectionHash::find(
 		const Kmer& key, bool& rc) const
 {
 	SequenceCollectionHash::const_iterator it = find(key);
-	if (it != m_data.end()) {
+	if (opt::ss || it != m_data.end()) {
 		rc = false;
 		return it;
 	} else {
@@ -190,6 +222,7 @@ bool SequenceCollectionHash::getSeqData(const Kmer& key,
 {
 	bool rc;
 	SequenceCollectionHash::const_iterator it = find(key, rc);
+	assert(!rc || !opt::ss);
 	if (it == m_data.end())
 		return false;
 	const KmerData data = it->second;
