@@ -169,6 +169,19 @@ class Histogram
 		return argMin((accumulator)ceil(p * sum()));
 	}
 
+	/** Return the expected value */
+	double expectedValue() const
+	{
+		double value = 0;
+		accumulator acc = sum();
+		for (Map::const_iterator it = m_map.begin();
+				it != m_map.end(); it++) {
+			value += (double)it->first * it->first
+				* it->second / acc;
+		}
+		return value;
+	}
+
 	/** Return the N50. */
 	T n50() const { return weightedPercentile(0.5); }
 
@@ -209,7 +222,8 @@ class Histogram
 	{
 		for (Map::iterator it = m_map.begin(); it != m_map.end();) {
 			if (m_map.count(it->first - 1) == 0
-					&& m_map.count(it->first + 1) == 0)
+					&& m_map.count(it->first + 1) == 0
+					&& m_map.size() > 1)
 				m_map.erase(it++);
 			else
 				++it;
@@ -319,8 +333,10 @@ static inline std::ostream& printContiguityStats(
 			<< "N80" << sep
 			<< "N50" << sep
 			<< "N20" << sep
+			<< "E-size" << sep
 			<< "max" << sep
-			<< "sum\n";
+			<< "sum" << sep
+			<< "name" << '\n';
 	}
 	unsigned n50 = h.n50();
 	out << toEng(h0.size()) << sep
@@ -341,6 +357,7 @@ static inline std::ostream& printContiguityStats(
 		<< toEng(h.weightedPercentile(1 - 0.8)) << sep
 		<< toEng(n50) << sep
 		<< toEng(h.weightedPercentile(1 - 0.2)) << sep
+		<< toEng((unsigned)h.expectedValue()) << sep
 		<< toEng(h.maximum()) << sep
 		<< toEng(sum);
 }
