@@ -4,6 +4,7 @@
 #include "config.h"
 #include "Sense.h"
 #include "Sequence.h"
+#include "Common/Hash.h"
 #include <cassert>
 #include <cstring> // for memcpy
 #include <stdint.h>
@@ -51,6 +52,8 @@ class Kmer
 	}
 
 	void reverseComplement();
+	bool isCanonical() const;
+	void canonicalize();
 
 	bool isPalindrome() const;
 	bool isPalindrome(extDirection dir) const;
@@ -122,9 +125,13 @@ static inline Kmer reverseComplement(const Kmer& seq)
 	return rc;
 }
 
-struct hashKmer
-{
-	size_t operator()(const Kmer& o) const { return o.getHashCode(); }
-};
+NAMESPACE_STD_HASH_BEGIN
+	template <> struct hash<Kmer> {
+		size_t operator()(const Kmer& kmer) const
+		{
+			return kmer.getHashCode();
+		}
+	};
+NAMESPACE_STD_HASH_END
 
 #endif
