@@ -5,6 +5,7 @@
 #include <iomanip>
 #include <sstream>
 #include <string>
+#include <cmath>
 
 /** Return the last character of s and remove it. */
 static inline char chop(std::string& s)
@@ -87,4 +88,70 @@ bool endsWith(const std::string& s, const std::string& suffix)
 			suffix.begin());
 }
 
+static inline 
+bool isReadNamePair(const std::string& name1, const std::string& name2) 
+{
+	assert(!name1.empty() && !name2.empty());
+
+	if (name1 == name2)
+		return true;
+
+	if (endsWith(name1,"/1") && endsWith(name2,"/2")) {
+		int len1 = name1.length();
+		int len2 = name2.length();
+		assert(len1 > 2 && len2 > 2);
+		return name1.compare(0, len1-2, name2, 0, len2-2) == 0;
+	}
+
+	return false;
+}
+
+static inline size_t SIToBytes(std::istringstream& iss)
+{
+	double size;
+	std::string units;
+
+	iss >> size;
+	if (iss.fail()) {
+		// not prefixed by a number
+		return 0;
+	}
+
+	iss >> units;
+	if (iss.fail() && iss.eof()) {
+		// no units given; clear fail flag
+		// and assume bytes
+		iss.clear(std::ios::eofbit);
+		return (size_t)ceil(size);
+	}
+
+	if (units.size() > 1) {
+		// unrecognized multichar suffix
+		iss.setstate(std::ios::failbit);
+		return 0;
+	}
+
+	switch(tolower(units[0])) {
+		case 'k':
+			size *= (size_t)1<<10; break;
+		case 'm':
+			size *= (size_t)1<<20; break;
+		case 'g':
+			size *= (size_t)1<<30; break;
+		case 't':
+			size *= (size_t)1<<40; break;
+		default:
+			iss.setstate(std::ios::failbit);
+			return 0;
+	}
+
+	return (size_t)ceil(size);
+}
+
+static inline size_t SIToBytes(const std::string& str)
+{
+	std::istringstream iss(str);
+	return SIToBytes(iss);
+}
+
 #endif
