@@ -45,8 +45,11 @@ class FastaReader {
 		/** Return whether this stream is at end-of-file. */
 		bool eof() const { return m_in.eof(); };
 
+		/** Return true if failbit or badbit of stream is set. */
+		bool fail() const { return m_in.fail(); };
+
 		/** Return whether this stream is good. */
-		operator void*() const { return m_in; }
+		operator const void*() const { return m_in ? this : NULL; }
 
 		/** Return the next character of this stream. */
 		int peek() { return m_in.peek(); }
@@ -133,6 +136,16 @@ struct FastaRecord
 			const Sequence& seq)
 		: id(id), comment(comment), anchor(0), seq(seq) { }
 
+	operator Sequence() const { return seq; }
+
+	FastaRecord& operator=(const std::string& s)
+	{
+		seq = s;
+		return *this;
+	}
+
+	size_t size() const { return seq.size(); }
+
 	friend FastaReader& operator >>(FastaReader& in, FastaRecord& o)
 	{
 		std::string q;
