@@ -32,10 +32,10 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Jared Simpson and Shaun Jackman.\n"
 "\n"
-"Copyright 2013 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2014 Canada's Michael Smith Genome Sciences Centre\n";
 
 static const char USAGE_MESSAGE[] =
-"Usage: " PROGRAM " [OPTION]... HIST [PAIR]\n"
+"Usage: " PROGRAM " -k<kmer> -s<seed-length> -n<npairs> [OPTION]... HIST [PAIR]\n"
 "Estimate distances between contigs using paired-end alignments.\n"
 "\n"
 " Arguments:\n"
@@ -157,6 +157,13 @@ static int estimateDistanceUsingMean(
 /** Global variable to track a recommended minAlign parameter */
 unsigned g_recMA;
 
+static struct {
+	/* Fragment stats are considered only for fragments aligning
+	 * to different contigs, and where the contig is >=opt::seedLen. */
+	unsigned total_frags;
+	unsigned dup_frags;
+} stats;
+
 /** Estimate the distance between two contigs.
  * @param numPairs [out] the number of pairs that agree with the
  * expected distribution
@@ -185,10 +192,15 @@ static int estimateDistance(unsigned len0, unsigned len1,
 	}
 
 	// Remove duplicate fragments.
+	unsigned orig = fragments.size();
 	sort(fragments.begin(), fragments.end());
 	fragments.erase(unique(fragments.begin(), fragments.end()),
 			fragments.end());
 	numPairs = fragments.size();
+	assert((int)orig - (int)numPairs >= 0);
+	stats.total_frags += orig;
+	stats.dup_frags += orig - numPairs;
+
 	if (numPairs < opt::npairs)
 		return INT_MIN;
 
@@ -318,33 +330,6 @@ static Histogram loadHist(const string& path)
 	return hist;
 }
 
-/** Read contig lengths from SAM headers. */
-static void readContigLengths(istream& in, vector<unsigned>& lengths)
-{
-	assert(in);
-	assert(lengths.empty());
-	assert(g_contigNames.empty());
-	for (string line; in.peek() == '@' && getline(in, line);) {
-		istringstream ss(line);
-		string type;
-		ss >> type;
-		if (type != "@SQ")
-			continue;
-
-		string s;
-		unsigned len;
-		ss >> expect(" SN:") >> s >> expect(" LN:") >> len;
-		assert(ss);
-
-		put(g_contigNames, lengths.size(), s);
-		lengths.push_back(len);
-	}
-	if (lengths.empty()) {
-		cerr << PROGRAM ": error: no @SQ records in the SAM header\n";
-		exit(EXIT_FAILURE);
-	}
-}
-
 /** Copy records from [it, last) to out and stop before alignments to
  * the next target sequence.
  * @param[in,out] it an input iterator
@@ -561,6 +546,18 @@ int main(int argc, char** argv)
 			break;
 		writeEstimates(out, records, contigLens, pmf);
 	}
+
+	if (opt::verbose > 0) {
+		float prop_dups = (float)100 * stats.dup_frags / stats.total_frags;
+		cerr << "Duplicate rate of spanning fragments: "
+			<< stats.dup_frags << "/"
+			<< stats.total_frags << " ("
+			<< setprecision(3) << prop_dups << "%)\n";
+		if (prop_dups > 0.5)
+			cerr << PROGRAM << ": warning: duplicate rate of fragments "
+				"spanning more than one contig is high.\n";
+	}
+
 	if (opt::verbose > 0 && g_recMA != opt::minAlign)
 		cerr << PROGRAM << ": warning: MLE will be more accurate if "
 			"l is decreased to " << g_recMA << ".\n";
