@@ -40,10 +40,10 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Tony Raymond.\n"
 "\n"
-"Copyright 2013 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2014 Canada's Michael Smith Genome Sciences Centre\n";
 
 static const char USAGE_MESSAGE[] =
-"Usage: " PROGRAM " [OPTION]... ADJ\n"
+"Usage: " PROGRAM " -k<kmer> [OPTION]... ADJ\n"
 "Remove short contigs that do not contribute any relevant\n"
 "information to the assembly.\n"
 "\n"
@@ -54,9 +54,12 @@ static const char USAGE_MESSAGE[] =
 " Options:\n"
 "\n"
 "  -k, --kmer=N          k-mer size\n"
+"      --SS              expect contigs to be oriented correctly\n"
+"      --no-SS           no assumption about contig orientation\n"
 "  -T, --island=N        remove islands shorter than N [0]\n"
 "  -t, --tip=N           remove tips shorter than N [0]\n"
 "  -l, --length=N        remove contigs shorter than N [0]\n"
+"  -L, --max-length=N    remove contigs longer than N [0]\n"
 "      --shim            remove filler contigs that only contribute\n"
 "                        to adjacency [default]\n"
 "      --no-shim         disable filler contigs removal\n"
@@ -80,6 +83,9 @@ static const char USAGE_MESSAGE[] =
 namespace opt {
 	unsigned k; // used by ContigProperties
 
+	/** Run a strand-specific RNA-Seq assembly. */
+	static int ss;
+
 	/** Remove island contigs less than this length. */
 	static unsigned minIslandLen = 0;
 
@@ -89,6 +95,9 @@ namespace opt {
 	/** Remove all contigs less than this length. */
 	static unsigned minLen = 0;
 
+	/** Remove all contigs more than this length. */
+	static unsigned maxLen = 0;
+
 	/** Remove short contigs that don't contribute any sequence. */
 	static int shim = 1;
 
@@ -111,7 +120,7 @@ namespace opt {
 	int format = ADJ; // used by ContigProperties
 }
 
-static const char shortopts[] = "g:i:r:k:l:m:t:T:v";
+static const char shortopts[] = "g:i:r:k:l:L:m:t:T:v";
 
 enum { OPT_HELP = 1, OPT_VERSION };
 
@@ -124,10 +133,13 @@ static const struct option longopts[] = {
 	{ "graph",         required_argument, NULL, 'g' },
 	{ "ignore",        required_argument, NULL, 'i' },
 	{ "remove",        required_argument, NULL, 'r' },
+	{ "SS",            no_argument,       &opt::ss, 1 },
+	{ "no-SS",         no_argument,       &opt::ss, 0 },
 	{ "kmer",          required_argument, NULL, 'k' },
 	{ "island",        required_argument, NULL, 'T' },
 	{ "tip",           required_argument, NULL, 't' },
 	{ "length",        required_argument, NULL, 'l' },
+	{ "max-length",    required_argument, NULL, 'L' },
 	{ "shim",          no_argument,       &opt::shim, 1 },
 	{ "no-shim",       no_argument,       &opt::shim, 0 },
 	{ "assemble",      no_argument,       &opt::assemble, 1 },
@@ -297,8 +309,6 @@ static void removeContigs(Graph& g, vector<vertex_descriptor>& sc)
 {
 	typedef graph_traits<Graph> GTraits;
 	typedef GTraits::vertex_descriptor V;
-	typedef GTraits::out_edge_iterator OEit;
-	typedef GTraits::in_edge_iterator IEit;
 
 	vector<vertex_descriptor> out;
 	out.reserve(sc.size());
@@ -353,7 +363,7 @@ static void findShortContigs(const Graph& g, const vector<bool>& seen,
 	typedef GTraits::vertex_iterator Vit;
 	Vit first, second;
 	tie(first, second) = vertices(g);
-	copy_if(first, second, back_inserter(sc),
+	::copy_if(first, second, back_inserter(sc),
 			!bind(Marked(g, seen), _1) && bind(removable, &g, _1));
 }
 
@@ -394,6 +404,21 @@ struct ShorterThanX : unary_function<vertex_descriptor, bool> {
 	}
 };
 
+struct LongerThanX : unary_function<vertex_descriptor, bool> {
+	const Graph& g;
+	const vector<bool>& seen;
+	size_t x;
+
+	LongerThanX(const Graph& g, const vector<bool>& seen, size_t x)
+		: g(g), seen(seen), x(x) { }
+
+	bool operator()(vertex_descriptor y) const
+	{
+		return g[y].length > x && !get(vertex_removed, g, y)
+			&& !seen[get(vertex_contig_index, g, y)];
+	}
+};
+
 static void removeShims(Graph& g, const vector<bool>& seen)
 {
 	if (opt::verbose > 0)
@@ -419,7 +444,8 @@ static void removeShims(Graph& g, const vector<bool>& seen)
 	}
 }
 
-static void removeShortContigs(Graph& g, const vector<bool>& seen)
+template <typename pred>
+static void removeContigs_if(Graph& g, pred p)
 {
 	typedef graph_traits<Graph> GTraits;
 	typedef GTraits::vertex_iterator Vit;
@@ -427,8 +453,7 @@ static void removeShortContigs(Graph& g, const vector<bool>& seen)
 	Vit first, second;
 	tie(first, second) = vertices(g);
 	vector<V> sc;
-	copy_if(first, second, back_inserter(sc),
-			ShorterThanX(g, seen, opt::minLen));
+	::copy_if(first, second, back_inserter(sc), p);
 	remove_vertex_if(g, sc.begin(), sc.end(), True<V>());
 	transform(sc.begin(), sc.end(), back_inserter(g_removed),
 			mem_fun_ref(&ContigNode::contigIndex));
@@ -458,6 +483,9 @@ int main(int argc, char** argv)
 		  case 'l':
 			arg >> opt::minLen;
 			break;
+		  case 'L':
+			arg >> opt::maxLen;
+			break;
 		  case 'm':
 			arg >> opt::minOverlap;
 			break;
@@ -581,9 +609,13 @@ int main(int argc, char** argv)
 
 	// Remove tips.
 	if (opt::minTipLen > 0) {
-		size_t s = g_removed.size();
-		pruneTips_if(g, back_inserter(g_removed),
-				ShorterThanX(g, seen, opt::minTipLen));
+		size_t s, prev;
+		s = g_removed.size();
+		do {
+			prev = g_removed.size();
+			pruneTips_if(g, back_inserter(g_removed),
+					ShorterThanX(g, seen, opt::minTipLen));
+		} while (prev < g_removed.size());
 		if (opt::verbose)
 			cerr << "Removed " << g_removed.size() - s
 				<< " tips.\n";
@@ -591,7 +623,11 @@ int main(int argc, char** argv)
 
 	// Remove short contigs.
 	if (opt::minLen > 0)
-		removeShortContigs(g, seen);
+		removeContigs_if(g, ShorterThanX(g, seen, opt::minLen));
+
+	// Remove long contigs.
+	if (opt::maxLen > 0)
+		removeContigs_if(g, LongerThanX(g, seen, opt::maxLen));
 
 	if (opt::verbose > 0) {
 		cerr << "Graph stats after:\n";
@@ -610,7 +646,11 @@ int main(int argc, char** argv)
 		size_t numContigs = num_vertices(g) / 2;
 		typedef vector<ContigPath> ContigPaths;
 		ContigPaths paths;
-		assemble(g, back_inserter(paths));
+		if (opt::ss)
+			assemble_stranded(g, back_inserter(paths));
+		else
+			assemble(g, back_inserter(paths));
+		g_contigNames.unlock();
 		for (ContigPaths::const_iterator it = paths.begin();
 				it != paths.end(); ++it) {
 			ContigNode u(numContigs + it - paths.begin(), false);
@@ -618,6 +658,7 @@ int main(int argc, char** argv)
 			put(vertex_name, g, u, name);
 			cout << name << '\t' << *it << '\n';
 		}
+		g_contigNames.lock();
 	}
 
 	// Output the updated adjacency graph.
