@@ -9,6 +9,8 @@
 #include "SAM.h"
 #include "StringUtil.h"
 #include "Uncompress.h"
+#include <boost/algorithm/string/join.hpp>
+#include <boost/tuple/tuple.hpp>
 #include <algorithm>
 #include <cassert>
 #include <cctype> // for toupper
@@ -25,6 +27,8 @@
 #endif
 
 using namespace std;
+using namespace boost;
+using namespace boost::algorithm;
 
 #define PROGRAM "abyss-map"
 
@@ -32,7 +36,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2013 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2014 Canada's Michael Smith Genome Sciences Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... QUERY... TARGET\n"
@@ -49,6 +53,11 @@ static const char USAGE_MESSAGE[] =
 "      --order             print alignments in the same order as\n"
 "                          read from QUERY\n"
 "      --no-order          print alignments ASAP [default]\n"
+"      --multi             Align unaligned segments of primary\n"
+"                          alignment\n"
+"      --no-multi          don't Align unaligned segments [default]\n"
+"      --SS                expect contigs to be oriented correctly\n"
+"      --no-SS             no assumption about contig orientation\n"
 "      --chastity          discard unchaste reads\n"
 "      --no-chastity       do not discard unchaste reads [default]\n"
 "  -v, --verbose           display verbose output\n"
@@ -67,9 +76,16 @@ namespace opt {
 	/** The number of parallel threads. */
 	static unsigned threads = 1;
 
+	/** Run a strand-specific RNA-Seq alignments. */
+	static int ss;
+
 	/** Identify duplicate and subsumed sequences. */
 	static bool dup = false;
 
+	/** Align unaligned segments of primary alignment. */
+	static int multi;
+
+	/** Ensure output order matches input order. */
 	static int order;
 
 	/** Verbose output. */
@@ -87,6 +103,10 @@ static const struct option longopts[] = {
 	{ "threads", required_argument, NULL, 'j' },
 	{ "order", no_argument, &opt::order, 1 },
 	{ "no-order", no_argument, &opt::order, 0 },
+	{ "multi", no_argument, &opt::multi, 1 },
+	{ "no-multi", no_argument, &opt::multi, 0 },
+	{ "SS", no_argument, &opt::ss, 1 },
+	{ "no-SS", no_argument, &opt::ss, 0 },
 	{ "verbose", no_argument, NULL, 'v' },
 	{ "chastity", no_argument, &opt::chastityFilter, 1 },
 	{ "no-chastity", no_argument, &opt::chastityFilter, 0 },
@@ -100,10 +120,46 @@ static struct {
 	unsigned unique;
 	unsigned multimapped;
 	unsigned unmapped;
+	unsigned suboptimal;
+	unsigned subunmapped;
 } g_count;
 
 typedef FMIndex::Match Match;
 
+#if SAM_SEQ_QUAL
+static string toXA(const FastaIndex& faIndex,
+		const FMIndex& fmIndex, const Match& m, bool rc,
+		unsigned qlength, unsigned seq_start)
+{
+	if (m.size() == 0)
+		return "";
+	FastaIndex::SeqPos seqPos = faIndex[fmIndex[m.l]];
+	string rname = seqPos.get<0>().id;
+	int pos = seqPos.get<1>() + 1;
+
+	// Set the mapq to the alignment score.
+	assert(m.qstart < m.qend);
+	unsigned matches = m.qend - m.qstart;
+
+	unsigned qstart = seq_start + m.qstart;
+	unsigned qend = m.qend + seq_start;
+	unsigned short flag = rc ? SAMAlignment::FREVERSE : 0;
+
+	ostringstream ss;
+	if (qstart > 0)
+		ss << qstart << 'S';
+	ss << matches << 'M';
+	if (qend < qlength)
+		ss << qlength - qend << 'S';
+	string cigar = ss.str();
+
+	stringstream xa_tag;
+	xa_tag << rname << ',' << pos << ',' << cigar << ",0," << flag;
+
+	return xa_tag.str();
+}
+#endif
+
 /** Return a SAM record of the specified match. */
 static SAMRecord toSAM(const FastaIndex& faIndex,
 		const FMIndex& fmIndex, const Match& m, bool rc,
@@ -126,7 +182,8 @@ static SAMRecord toSAM(const FastaIndex& faIndex,
 		// Set the mapq to the alignment score.
 		assert(m.qstart < m.qend);
 		unsigned matches = m.qend - m.qstart;
-		a.mapq = m.size() > 1 ? 0 : min(matches, 255U);
+		assert (m.num != 0);
+		a.mapq = m.size() > 1 || m.num > 1 ? 0 : min(matches, 254U);
 
 		ostringstream ss;
 		if (m.qstart > 0)
@@ -186,8 +243,12 @@ static void printDuplicates(const Match& m, const Match& rcm,
 		const FastqRecord& rec)
 {
 	size_t myLen = m.qspan();
-	size_t maxLen = max(getMaxLen(m, faIndex, fmIndex),
-			getMaxLen(rcm, faIndex, fmIndex));
+	size_t maxLen;
+	if (opt::ss)
+		maxLen = getMaxLen(m, faIndex, fmIndex);
+	else
+		maxLen = max(getMaxLen(m, faIndex, fmIndex),
+				getMaxLen(rcm, faIndex, fmIndex));
 	if (myLen < maxLen) {
 #pragma omp atomic
 		g_count.multimapped++;
@@ -199,8 +260,12 @@ static void printDuplicates(const Match& m, const Match& rcm,
 		return;
 	}
 	size_t myPos = getMyPos(m, faIndex, fmIndex, rec.id);
-	size_t minPos = min(getMinPos(m, maxLen, faIndex, fmIndex),
-			getMinPos(rcm, maxLen, faIndex, fmIndex));
+	size_t minPos;
+	if (opt::ss)
+		minPos = getMinPos(m, maxLen, faIndex, fmIndex);
+	else
+		minPos = min(getMinPos(m, maxLen, faIndex, fmIndex),
+				getMinPos(rcm, maxLen, faIndex, fmIndex));
 	if (myPos > minPos) {
 #pragma omp atomic
 		g_count.multimapped++;
@@ -215,6 +280,21 @@ static void printDuplicates(const Match& m, const Match& rcm,
 	return;
 }
 
+pair<Match, Match> findMatch(const FMIndex& fmIndex, const string& seq)
+{
+	Match m = fmIndex.find(seq,
+			opt::dup ? seq.length() : opt::k);
+
+	string rcqseq = reverseComplement(seq);
+	Match rcm;
+	if (opt::ss)
+		rcm = fmIndex.find(rcqseq,
+				opt::dup ? rcqseq.length() : opt::k);
+	else
+		rcm = fmIndex.find(rcqseq,
+				opt::dup ? rcqseq.length() : m.qspan());
+	return make_pair(m, rcm);
+}
 
 static queue<string> g_pq;
 
@@ -228,21 +308,68 @@ static void find(const FastaIndex& faIndex, const FMIndex& fmIndex,
 		exit(EXIT_FAILURE);
 	}
 
-	Match m = fmIndex.find(rec.seq,
-			opt::dup ? rec.seq.length() : opt::k);
-
-	string rcqseq = reverseComplement(rec.seq);
-	Match rcm = fmIndex.find(rcqseq,
-			opt::dup ? rcqseq.length() : m.qspan());
+	Match m, rcm;
+	tie(m, rcm) = findMatch(fmIndex, rec.seq);
 
 	if (opt::dup) {
 		printDuplicates(m, rcm, faIndex, fmIndex, rec);
 		return;
 	}
 
-	bool rc = rcm.qspan() > m.qspan();
+	bool rc;
+	if (opt::ss) {
+		rc = rec.id.size() > 2
+			&& rec.id.substr(rec.id.size()-2) == "/1";
+		bool prc = rcm.qspan() > m.qspan();
+		if (prc != rc && ((rc && rcm.size() > 0)
+					|| (!rc && m.size() > 0)))
+#pragma omp atomic
+			g_count.suboptimal++;
+		if (prc != rc && ((rc && rcm.size() == 0 && m.size() > 0)
+				|| (!rc && m.size() == 0 && rcm.size() > 0)))
+#pragma omp atomic
+			g_count.subunmapped++;
+	} else {
+		rc = rcm.qspan() > m.qspan();
 
-	SAMRecord sam = toSAM(faIndex, fmIndex, rc ? rcm : m, rc,
+		// if both matches are the same length, sum up the number of times
+		// each were seen.
+		if (rcm.qspan() == m.qspan())
+			rc ? rcm.num += m.num : m.num += rcm.num;
+	}
+
+	vector<string> alts;
+	Match mm = rc ? rcm : m;
+	string mseq = rc ? reverseComplement(rec.seq) : rec.seq;
+
+#if SAM_SEQ_QUAL
+	if (opt::multi) {
+		if (mm.qstart > 0) {
+			string seq = mseq.substr(0, mm.qstart);
+			Match m1, rcm1;
+			tie(m1, rcm1) = findMatch(fmIndex, seq);
+			bool rc1 = rcm1.qspan() > m1.qspan();
+			string xa = toXA(faIndex, fmIndex, rc1 ? rcm1 : m1,
+					rc ^ rc1, mseq.size(), 0);
+			if (xa != "")
+				alts.push_back(xa);
+		}
+
+		if (mm.qend < mseq.size()) {
+			string seq =
+				mseq.substr(mm.qend, mseq.length() - mm.qend);
+			Match m2, rcm2;
+			tie(m2, rcm2) = findMatch(fmIndex, seq);
+			bool rc2 = rcm2.qspan() > m2.qspan();
+			string xa = toXA(faIndex, fmIndex, rc2 ? rcm2 : m2,
+					rc ^ rc2, mseq.size(), mm.qend);
+			if (xa != "")
+				alts.push_back(xa);
+		}
+	}
+#endif
+
+	SAMRecord sam = toSAM(faIndex, fmIndex, mm, rc,
 			rec.seq.size());
 	if (rec.id[0] == '@') {
 		cerr << PROGRAM ": error: "
@@ -253,18 +380,12 @@ static void find(const FastaIndex& faIndex, const FMIndex& fmIndex,
 	sam.qname = rec.id;
 
 #if SAM_SEQ_QUAL
-	sam.seq = rc ? rcqseq : rec.seq;
+	sam.seq = mseq;
 	sam.qual = rec.qual.empty() ? "*" : rec.qual;
 	if (rc)
 		reverse(sam.qual.begin(), sam.qual.end());
 #endif
 
-	if (m.qstart == rec.seq.size() - rcm.qend
-			&& m.qspan() == rcm.qspan()) {
-		// This matching sequence maps to both strands.
-		sam.mapq = 0;
-	}
-
 	bool print = opt::order == 0;
 	do {
 #pragma omp critical(cout)
@@ -276,7 +397,12 @@ static void find(const FastaIndex& faIndex, const FMIndex& fmIndex,
 					g_pq.pop();
 			}
 			if (print) {
-				cout << sam << '\n';
+				cout << sam;
+#if SAM_SEQ_QUAL
+				if (alts.size() > 0)
+					cout << "\tXA:Z:" << join(alts, ";");
+#endif
+				cout << '\n';
 				assert_good(cout, "stdout");
 			}
 		}
@@ -412,6 +538,17 @@ int main(int argc, char** argv)
 		}
 	}
 
+#ifndef SAM_SEQ_QUAL
+# define SAM_SEQ_QUAL 0
+#endif
+
+	if (opt::multi && !SAM_SEQ_QUAL) {
+		cerr << PROGRAM ": multiple alignments not supported with "
+			"this install. Recompile ABySS with `./configure "
+			"--enable-samseqqual'.\n";
+		die = true;
+	}
+
 	if (argc - optind < 2) {
 		cerr << PROGRAM ": missing arguments\n";
 		die = true;
@@ -512,6 +649,13 @@ int main(int argc, char** argv)
 			<< "Mapped " << unique << " of " << total
 			<< " reads uniquely (" << (float)100 * unique / total
 			<< "%)\n";
+		if (opt::ss)
+			cerr << "Mapped " << g_count.suboptimal
+				<< " (" << (float)100 * g_count.suboptimal / total << "%)"
+				<< " reads to the opposite strand of the optimal mapping.\n"
+				<< "Made " << g_count.subunmapped << " ("
+				<< (float)100 * g_count.subunmapped / total << "%)"
+				<< " unmapped suboptimal decisions.\n";
 	}
 
 	cout.flush();
