@@ -42,10 +42,10 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Jared Simpson and Shaun Jackman.\n"
 "\n"
-"Copyright 2013 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2014 Canada's Michael Smith Genome Sciences Centre\n";
 
 static const char USAGE_MESSAGE[] =
-"Usage: " PROGRAM " [OPTION]... LEN PATH\n"
+"Usage: " PROGRAM " -k<kmer> [OPTION]... LEN PATH\n"
 "Merge sequences of contigs IDs.\n"
 "\n"
 " Arguments:\n"
