@@ -28,6 +28,7 @@
 #include <vector>
 
 using namespace std;
+using boost::tie;
 
 #define PROGRAM "PathConsensus"
 
@@ -35,10 +36,10 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman and Rong She.\n"
 "\n"
-"Copyright 2013 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2014 Canada's Michael Smith Genome Sciences Centre\n";
 
 static const char USAGE_MESSAGE[] =
-"Usage: " PROGRAM " [OPTION]... FASTA ADJ PATH\n"
+"Usage: " PROGRAM " -k<kmer> -s<consensus.fa> -o<out.path> [OPTION]... FASTA ADJ PATH\n"
 "Align sequences of ambiguous paths and output a consensus\n"
 "sequence.\n"
 "\n"
@@ -145,6 +146,7 @@ struct AmbPathConstraint {
 	}
 };
 
+typedef ContigGraph<DirectedGraph<ContigProperties, Distance> > Graph;
 typedef ContigPath Path;
 typedef vector<Path> ContigPaths;
 typedef map<AmbPathConstraint, ContigPath> AmbPath2Contig;
@@ -188,8 +190,6 @@ static int getDistance(const Graph& g,
 static ContigPaths readPaths(const string& inPath,
 	vector<string>& ids, vector<bool>& isAmb)
 {
-	typedef graph_traits<Graph>::vertex_descriptor V;
-
 	assert(ids.empty());
 	assert(isAmb.empty());
 	assert(g_ambpath_contig.empty());
@@ -600,9 +600,11 @@ static ContigPath alignMulti(const Graph& g,
 		return ContigPath();
 	}
 
-	string alignment;
-	unsigned matches;
-	Sequence consensus = dialign(amb_seqs, alignment, matches);
+	unsigned matches, consensusSize;
+	NWAlignment alignment;
+	tie(matches, consensusSize) = align(amb_seqs, alignment);
+	string consensus = alignment.consensus();
+
 	if (opt::verbose > 2)
 	   	cerr << alignment << consensus << '\n';
 	float identity = (float)matches / consensus.size();
