@@ -79,6 +79,7 @@ void NetworkSequenceCollection::run()
 				logger(0) << "Loaded " << m_data.size()
 					<< " k-mer.\n";
 				assert(!m_data.empty());
+				m_data.setDeletedKey();
 				m_data.shrink();
 				m_comm.reduce(m_data.size());
 
@@ -437,6 +438,7 @@ void NetworkSequenceCollection::runControl()
 				logger(0) << "Loaded " << m_data.size()
 					<< " k-mer.\n";
 				assert(!m_data.empty() || opt::numProc >= DEDICATE_CONTROL_AT);
+				m_data.setDeletedKey();
 				m_data.shrink();
 				size_t numLoaded = m_comm.reduce(m_data.size());
 				cout << "Loaded " << numLoaded << " k-mer. "
@@ -1196,7 +1198,8 @@ processBranchesAssembly(ISequenceCollection* seqCollection,
 			assert(branch.getState() == BS_NOEXT
 					|| branch.getState() == BS_AMBI_SAME
 					|| branch.getState() == BS_AMBI_OPP);
-			if (branch.isCanonical()) {
+			if ((opt::ss && branch.getDirection() == SENSE)
+					|| (!opt::ss && branch.isCanonical())) {
 				assembledContigs++;
 				assembledKmer += branch.size();
 				assembleContig(seqCollection, fileWriter, branch,
