@@ -17,6 +17,7 @@
 #include <iterator>
 #include <sstream>
 #include <string>
+#include <boost/unordered_map.hpp>
 
 using namespace std;
 
@@ -26,7 +27,7 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2013 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2014 Canada's Michael Smith Genome Sciences Centre\n";
 
 static const char USAGE_MESSAGE[] =
 "Usage: " PROGRAM " [OPTION]... [FILE]...\n"
@@ -38,6 +39,9 @@ static const char USAGE_MESSAGE[] =
 "\n"
 "      --no-qname        set the qname to * [default]\n"
 "      --qname           do not alter the qname\n"
+"      --all             print all alignments\n"
+"      --diff            print alignments that align to different\n"
+"                        contigs [default]\n"
 "  -l, --min-align=N     the minimal alignment size [1]\n"
 "  -s, --same=SAME       write properly-paired reads to this file\n"
 "  -h, --hist=FILE       write the fragment size histogram to FILE\n"
@@ -54,6 +58,7 @@ namespace opt {
 	static string covPath;
 	static int qname;
 	static int verbose;
+	static int print_all;
 }
 
 static const char shortopts[] = "h:c:l:s:v";
@@ -63,6 +68,8 @@ enum { OPT_HELP = 1, OPT_VERSION };
 static const struct option longopts[] = {
 	{ "qname",     no_argument,       &opt::qname, 1 },
 	{ "no-qname",  no_argument,       &opt::qname, 0 },
+	{ "all",       no_argument,       &opt::print_all, 1 },
+	{ "diff",      no_argument,       &opt::print_all, 0 },
 	{ "min-align", required_argument, NULL, 'l' },
 	{ "hist",      required_argument, NULL, 'h' },
 	{ "cov",       required_argument, NULL, 'c' },
@@ -123,7 +130,7 @@ static void handlePair(SAMRecord& a0, SAMRecord& a1)
 		stats.numDifferent++;
 		// Set the mapping quality of both reads to their minimum.
 		a0.mapq = a1.mapq = min(a0.mapq, a1.mapq);
-		if (!opt::histPath.empty())
+		if (!opt::print_all)
 			cout << a0 << '\n' << a1 << '\n';
 	} else if (a0.isReverse() == a1.isReverse()) {
 		// Same target, FF orientation.
@@ -141,16 +148,16 @@ static void handlePair(SAMRecord& a0, SAMRecord& a1)
 		}
 	}
 
-	if (opt::histPath.empty()) {
+	if (opt::print_all) {
 		cout << a0 << '\n' << a1 << '\n';
 		assert(cout.good());
 	}
 }
 
 #if SAM_SEQ_QUAL
-typedef unordered_map<string, SAMRecord> Alignments;
+typedef boost::unordered_map<string, SAMRecord> Alignments;
 #else
-typedef unordered_map<string, SAMAlignment> Alignments;
+typedef boost::unordered_map<string, SAMAlignment> Alignments;
 #endif
 
 static void printProgress(const Alignments& map)
@@ -184,7 +191,14 @@ static void handleAlignment(SAMRecord& sam, Alignments& map)
 		SAMRecord a0(it.first->second, it.first->first);
 #endif
 		handlePair(a0, sam);
+
+#include <boost/version.hpp>
+#if BOOST_VERSION >= 104300
+		// erase() is slow on older compilers so we use quick_erase()
+		map.quick_erase(it.first);
+#else
 		map.erase(it.first);
+#endif
 	}
 	stats.alignments++;
 	printProgress(map);
@@ -282,15 +296,23 @@ static string percent(size_t x, size_t n)
 	return ss.str();
 }
 
-/** Print statistics of the specified histogram. */
-static void printHistogramStats(const Histogram& h)
+/** Print statistics of the specified histogram. h not passed by
+  * reference because we want to make a copy
+ **/
+static void printHistogramStats(Histogram h)
 {
+	unsigned n_orig = h.size();
+	h.eraseNegative();
+	h.removeNoise();
+	h.removeOutliers();
+	h = h.trimFraction(0.0001);
 	cerr << "Stats mean: " << setprecision(4) << h.mean() << " "
 		"median: " << setprecision(4) << h.median() << " "
 		"sd: " << setprecision(4) << h.sd() << " "
 		"n: " << h.size() << " "
 		"min: " << h.minimum() << " "
-		"max: " << h.maximum() << '\n'
+		"max: " << h.maximum() << " "
+		"ignored: " << n_orig - h.size() << '\n'
 		<< h.barplot() << endl;
 }
 
@@ -347,7 +369,7 @@ int main(int argc, char* const* argv)
 		cerr << "Read " << stats.alignments << " alignments" << endl;
 
 	// Print the unpaired alignments.
-	if (opt::histPath.empty()) {
+	if (opt::print_all) {
 		for (Alignments::iterator it = alignments.begin();
 				it != alignments.end(); it++) {
 #if SAM_SEQ_QUAL
@@ -355,6 +377,7 @@ int main(int argc, char* const* argv)
 #else
 			SAMRecord a0(it->second, it->first);
 #endif
+			a0.noMate();
 			cout << a0 << '\n';
 			assert(cout.good());
 		}
@@ -399,22 +422,14 @@ int main(int argc, char* const* argv)
 
 		// Print the statistics of the forward-reverse distribution.
 		if ((float)numFR / numTotal > 0.001) {
-			Histogram h = g_histogram;
-			h.eraseNegative();
-			h.removeNoise();
-			h.removeOutliers();
 			cerr << "FR ";
-			printHistogramStats(h.trimFraction(0.0001));
+			printHistogramStats(g_histogram);
 		}
 
 		// Print the statistics of the reverse-forward distribution.
 		if ((float)numRF / numTotal > 0.001) {
-			Histogram h = g_histogram.negate();
-			h.eraseNegative();
-			h.removeNoise();
-			h.removeOutliers();
 			cerr << "RF ";
-			printHistogramStats(h.trimFraction(0.0001));
+			printHistogramStats(g_histogram.negate());
 		}
 	}
 
