@@ -37,10 +37,10 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Shaun Jackman.\n"
 "\n"
-"Copyright 2013 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2014 Canada's Michael Smith Genome Sciences Centre\n";
 
 static const char USAGE_MESSAGE[] =
-"Usage: " PROGRAM " [OPTION]... FASTA|OVERLAP DIST...\n"
+"Usage: " PROGRAM " -k<kmer> [OPTION]... FASTA|OVERLAP DIST...\n"
 "Scaffold contigs using the distance estimate graph.\n"
 "\n"
 " Arguments:\n"
@@ -57,6 +57,10 @@ static const char USAGE_MESSAGE[] =
 "                        that maximizes the scaffold N50.\n"
 "  -k, --kmer=N          length of a k-mer\n"
 "      --min-gap=N       minimum scaffold gap length to output [50]\n"
+"      --complex         remove complex transitive edges\n"
+"      --no-complex      don't remove complex transitive edges [default]\n"
+"      --SS              expect contigs to be oriented correctly\n"
+"      --no-SS           no assumption about contig orientation [default]\n"
 "  -o, --out=FILE        write the paths to FILE\n"
 "  -g, --graph=FILE      write the graph to FILE\n"
 "  -v, --verbose         display verbose output\n"
@@ -84,16 +88,22 @@ namespace opt {
 	/** Write the graph to this file. */
 	static string graphPath;
 
+	/** Run a strand-specific RNA-Seq assembly. */
+	static int ss;
+
 	/** Verbose output. */
 	int verbose; // used by PopBubbles
 
 	/** Output format */
 	int format = DOT; // used by DistanceEst
+
+	/** Remove complex transitive edges */
+	static int comp_trans;
 }
 
 static const char shortopts[] = "g:k:n:o:s:v";
 
-enum { OPT_HELP = 1, OPT_VERSION, OPT_MIN_GAP };
+enum { OPT_HELP = 1, OPT_VERSION, OPT_MIN_GAP, OPT_COMP };
 
 static const struct option longopts[] = {
 	{ "graph",       no_argument,       NULL, 'g' },
@@ -102,6 +112,10 @@ static const struct option longopts[] = {
 	{ "npairs",      required_argument, NULL, 'n' },
 	{ "out",         required_argument, NULL, 'o' },
 	{ "seed-length", required_argument, NULL, 's' },
+	{ "complex",     no_argument, &opt::comp_trans, 1 },
+	{ "no-complex",  no_argument, &opt::comp_trans, 0 },
+	{ "SS",          no_argument,       &opt::ss, 1 },
+	{ "no-SS",       no_argument,       &opt::ss, 0 },
 	{ "verbose",     no_argument,       NULL, 'v' },
 	{ "help",        no_argument,       NULL, OPT_HELP },
 	{ "version",     no_argument,       NULL, OPT_VERSION },
@@ -142,8 +156,6 @@ struct PoorSupport {
 static void filterGraph(Graph& g, unsigned minContigLength)
 {
 	typedef graph_traits<Graph> GTraits;
-	typedef GTraits::edge_descriptor E;
-	typedef GTraits::edge_iterator Eit;
 	typedef GTraits::vertex_descriptor V;
 	typedef GTraits::vertex_iterator Vit;
 
@@ -258,8 +270,6 @@ static void resolveForks(Graph& g, const Graph& g0)
  */
 static void pruneTips(Graph& g)
 {
-	typedef graph_traits<Graph>::vertex_descriptor V;
-
 	/** Identify the tips. */
 	size_t n = 0;
 	pruneTips(g, CountingOutputIterator(n));
@@ -565,7 +575,12 @@ unsigned scaffold(const Graph& g0, unsigned minContigLength,
 	removeRepeats(g);
 
 	// Remove transitive edges.
-	unsigned numTransitive = remove_transitive_edges(g);
+	unsigned numTransitive;
+	if (opt::comp_trans)
+		numTransitive = remove_complex_transitive_edges(g);
+	else
+		numTransitive = remove_transitive_edges(g);
+
 	if (opt::verbose > 0) {
 		cerr << "Removed " << numTransitive << " transitive edges.\n";
 		printGraphStats(cerr, g);
@@ -595,7 +610,7 @@ unsigned scaffold(const Graph& g0, unsigned minContigLength,
 
 	// Assemble the paths.
 	ContigPaths paths;
-	assembleDFS(g, back_inserter(paths));
+	assembleDFS(g, back_inserter(paths), opt::ss);
 	sort(paths.begin(), paths.end());
 	if (opt::verbose > 0) {
 		unsigned n = 0;
@@ -731,7 +746,7 @@ int main(int argc, char** argv)
 	remove_edge_if(InvalidEdge(g), static_cast<DG&>(g));
 	unsigned numRemoved = numBefore - num_edges(g);
 	if (numRemoved > 0)
-		cerr << "waning: Removed "
+		cerr << "warning: Removed "
 			<< numRemoved << " invalid edges.\n";
 
 	if (opt::minContigLengthEnd == 0) {
