@@ -30,10 +30,10 @@ static const char VERSION_MESSAGE[] =
 PROGRAM " (" PACKAGE_NAME ") " VERSION "\n"
 "Written by Jared Simpson and Shaun Jackman.\n"
 "\n"
-"Copyright 2013 Canada's Michael Smith Genome Science Centre\n";
+"Copyright 2014 Canada's Michael Smith Genome Sciences Centre\n";
 
 static const char USAGE_MESSAGE[] =
-"Usage: " PROGRAM " [OPTION]... ADJ DIST\n"
+"Usage: " PROGRAM " -k<kmer> -o<out.path> [OPTION]... ADJ DIST\n"
 "Find paths through contigs using distance estimates.\n"
 "\n"
 " Arguments:\n"
@@ -94,6 +94,7 @@ static const struct option longopts[] = {
 	{ NULL, 0, NULL, 0 }
 };
 
+typedef ContigGraph<DirectedGraph<ContigProperties, Distance> > Graph;
 static void generatePathsThroughEstimates(const Graph& g,
 		const string& estPath);
 
@@ -258,7 +259,7 @@ struct ComparePathLength
 		unsigned lenA = calculatePathLength(m_g, m_origin, a);
 		unsigned lenB = calculatePathLength(m_g, m_origin, b);
 		return lenA < lenB
-			|| lenA == lenB && a.size() < b.size();
+			|| (lenA == lenB && a.size() < b.size());
 	}
   private:
 	const Graph& m_g;
